/*
 * Copyright 2016 (c) Anna Schumaker.
 */
#ifndef OCARINA_CORE_PLAYLISTS_SYSTEM_H
#define OCARINA_CORE_PLAYLISTS_SYSTEM_H
#include <core/playlists/type.h>


enum sys_playlist_t {
	SYS_PL_FAVORITES,     /* Songs that the user likes.              */
	SYS_PL_HIDDEN,        /* Songs that the user has hidden.         */
	SYS_PL_COLLECTION,    /* Songs that have not been hidden.        */
	SYS_PL_HISTORY,       /* Songs that have just been played.       */
	SYS_PL_UNPLAYED,      /* Songs that have not been played yet.    */
	SYS_PL_MOST_PLAYED,   /* Songs with an above average play count. */
	SYS_PL_LEAST_PLAYED,  /* Songs with a below average play count.  */
	SYS_PL_NUM_PLAYLISTS, /* Number of system playlists.             */
};

struct sys_playlist {
	struct playlist spl_playlist;

	void (*spl_init)(struct playlist *, unsigned int, struct queue_ops *);
	bool (*spl_add)(struct playlist *, struct track *);
	bool (*spl_remove)(struct playlist *, struct track *);
	bool (*spl_update)(struct playlist *, struct track *);
	void (*spl_set_flag)(struct playlist *, enum queue_flags, bool);
	void (*spl_sort)(struct playlist *, enum compare_t, bool);
};


/* System playlist type. */
extern struct playlist_type pl_system;


/* Called to initialize system playlists. */
void pl_system_init(struct queue_ops *);

/* Called to deinitialize system playlists. */
void pl_system_deinit();


/* Called to tell system playlists about a new track. */
void pl_system_new_track(struct track *);

/* Called to tell system playlists that a track is getting deleted. */
void pl_system_delete_track(struct track *);

#endif /* OCARINA_CORE_PLAYLISTS_SYSTEM_H */
